/*
 ***********************************************************************************************************************
 *
 *  Copyright (c) 2018-2025 Advanced Micro Devices, Inc. All Rights Reserved.
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in all
 *  copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 *  SOFTWARE.
 *
 **********************************************************************************************************************/
#pragma once

#if GPURT_BUILD_RTIP3_1

// Defining arrays of data at the time of struct declaration has to be done differnetly
// for hlsl and cpp, so since hlsl doesn't need the data inline (it gets put into a const buffer)
// the definition of this struct is limited to the cpp side
#ifndef __cplusplus
struct ObbDataSource
{
    static const float ObbAxesLut[13 * 4];
    static const float ObbKdopDirections[13 * 4];
    static uint ObbKdopFacets[48 * 4];
    static const uint ObbMatrixIndexLUT[1024 * 4];
};
#else
static struct ObbDataSource
{
    // OBB axes. 13 total.
    const float ObbAxesLut[13 * 4] = {
        1.0f,       0.0f,       0.0f,       0.f, // 0
        0.0f,       1.0f,       0.0f,       0.f, // 1
        0.0f,       0.0f,       1.0f,       0.f, // 2
        0.0f,       +0.707107f, +0.707107f, 0.f, // 3
        0.0f,       -0.707107f, +0.707107f, 0.f, // 4
        +0.707107f, 0.0f,       +0.707107f, 0.f, // 5
        +0.707107f, 0.0f,       -0.707107f, 0.f, // 6
        +0.707107f, +0.707107f, 0.0f,       0.f, // 7
        -0.707107f, +0.707107f, 0.0f,       0.f, // 8
        +0.57735f,  +0.57735f,  +0.57735f,  0.f, // 9
        -0.57735f,  +0.57735f,  +0.57735f,  0.f, // 10
        +0.57735f,  -0.57735f,  +0.57735f,  0.f, // 11
        -0.57735f,  -0.57735f,  +0.57735f,  0.f  // 12
    };

    // OBB K-DOP Directions where K = 26, 56, 98:
    const float ObbKdopDirections[13 * 4] = {
        // K-DOP Directions where N = 2, K = 26
        +0.5773502588f, -0.5773502588f, -0.5773502588f, +0.0000000000f, // 0
        +0.7071067691f, +0.0000000000f, -0.7071067691f, +0.0000000000f, // 1
        +0.5773502588f, +0.5773502588f, -0.5773502588f, +0.0000000000f, // 2
        +0.7071067691f, -0.7071067691f, +0.0000000000f, +0.0000000000f, // 3
        +1.0000000000f, +0.0000000000f, +0.0000000000f, +0.0000000000f, // 4
        +0.7071067691f, +0.7071067691f, +0.0000000000f, +0.0000000000f, // 5
        +0.5773502588f, -0.5773502588f, +0.5773502588f, +0.0000000000f, // 6
        +0.7071067691f, +0.0000000000f, +0.7071067691f, +0.0000000000f, // 7
        +0.5773502588f, +0.5773502588f, +0.5773502588f, +0.0000000000f, // 8
        +0.0000000000f, +0.7071067691f, -0.7071067691f, +0.0000000000f, // 9
        +0.0000000000f, +1.0000000000f, +0.0000000000f, +0.0000000000f, // 10
        +0.0000000000f, +0.7071067691f, +0.7071067691f, +0.0000000000f, // 11
        +0.0000000000f, +0.0000000000f, +1.0000000000f, +0.0000000000f, // 12
    };

    // Apex point map facets, antipodal pair triplets next to each other in memory.
    uint ObbKdopFacets[48 * 4] = {
        // K-DOP Facets where N = 2, F = 48
        0, 2, 8, 0, // 0
        1, 3, 9, 0, // 1
        0, 6, 8, 0, // 2
        1, 7, 9, 0, // 3
        2, 4, 10, 0, // 4
        3, 5, 11, 0, // 5
        2, 8, 10, 0, // 6
        3, 9, 11, 0, // 7
        6, 8, 14, 0, // 8
        7, 9, 15, 0, // 9
        6, 12, 14, 0, // 10
        7, 13, 15, 0, // 11
        8, 10, 16, 0, // 12
        9, 11, 17, 0, // 13
        8, 14, 16, 0, // 14
        9, 15, 17, 0, // 15
        13, 7, 20, 0, // 16
        12, 6, 21, 0, // 17
        13, 18, 20, 0, // 18
        12, 19, 21, 0, // 19
        7, 1, 22, 0, // 20
        6, 0, 23, 0, // 21
        7, 20, 22, 0, // 22
        6, 21, 23, 0, // 23
        18, 20, 10, 0, // 24
        19, 21, 11, 0, // 25
        18, 4, 10, 0, // 26
        19, 5, 11, 0, // 27
        20, 22, 16, 0, // 28
        21, 23, 17, 0, // 29
        20, 10, 16, 0, // 30
        21, 11, 17, 0, // 31
        5, 19, 24, 0, // 32
        4, 18, 25, 0, // 33
        5, 3, 24, 0, // 34
        4, 2, 25, 0, // 35
        19, 12, 14, 0, // 36
        18, 13, 15, 0, // 37
        19, 24, 14, 0, // 38
        18, 25, 15, 0, // 39
        3, 24, 22, 0, // 40
        2, 25, 23, 0, // 41
        3, 1, 22, 0, // 42
        2, 0, 23, 0, // 43
        24, 14, 16, 0, // 44
        25, 15, 17, 0, // 45
        24, 22, 16, 0, // 46
        25, 23, 17, 0, // 47
    };

    // Normalized axis-angle matrix representaton to OBB matrix index LUT. See MatrixToOBBIndex()
    const uint ObbMatrixIndexLUT[1024 * 4] = {
        57, 59, 61, 62, 62, 60, 58, 18, 18, 59, 61, 62, 62, 60, 58, 56,
        57, 59, 61, 84, 29, 60, 58, 72, 96, 59, 61, 37, 92, 60, 58, 56,
        57, 59, 84, 84, 27, 54, 74, 72, 96, 98, 46, 35, 92, 92, 58, 56,
        57, 59, 83, 83, 27, 54, 74, 16, 16, 98, 46, 35, 91, 91, 58, 56,
        127, 57, 83, 86, 27, 76, 77, 18, 18, 101, 100, 35, 94, 91, 56, 127,
        127, 1, 67, 86, 9, 59, 38, 27, 27, 70, 50, 17, 94, 43, 0, 127,
        127, 1, 67, 65, 9, 59, 84, 27, 27, 76, 50, 17, 41, 43, 0, 127,
        127, 1, 67, 95, 32, 98, 67, 10, 10, 59, 83, 25, 78, 43, 0, 127,
        127, 1, 42, 95, 32, 98, 67, 10, 10, 59, 83, 25, 78, 59, 0, 127,
        127, 1, 42, 40, 16, 51, 93, 34, 34, 101, 67, 9, 57, 59, 0, 127,
        127, 1, 42, 79, 16, 51, 62, 34, 34, 30, 67, 9, 103, 59, 0, 127,
        127, 64, 74, 79, 26, 85, 84, 18, 18, 92, 93, 34, 103, 98, 65, 127,
        64, 66, 74, 74, 26, 46, 83, 17, 17, 91, 54, 34, 98, 98, 67, 65,
        64, 66, 77, 77, 26, 46, 83, 81, 89, 91, 54, 34, 101, 101, 67, 65,
        64, 66, 68, 77, 28, 69, 67, 81, 89, 66, 68, 36, 101, 69, 67, 65,
        64, 66, 68, 70, 70, 69, 67, 18, 18, 66, 68, 70, 70, 69, 67, 65,
        57, 59, 61, 85, 53, 60, 58, 97, 73, 59, 61, 44, 93, 60, 58, 56,
        57, 59, 61, 37, 29, 60, 58, 56, 57, 59, 61, 37, 29, 60, 58, 56,
        57, 59, 61, 84, 29, 54, 58, 72, 96, 59, 46, 37, 92, 60, 58, 56,
        127, 57, 46, 83, 27, 66, 42, 40, 32, 34, 26, 35, 91, 30, 56, 127,
        127, 103, 67, 67, 10, 87, 82, 53, 60, 74, 79, 18, 43, 43, 102, 127,
        127, 103, 67, 67, 10, 26, 82, 91, 91, 74, 42, 18, 43, 43, 102, 127,
        127, 49, 30, 90, 34, 26, 35, 33, 64, 66, 42, 27, 75, 70, 48, 127,
        127, 49, 51, 90, 34, 70, 99, 33, 64, 82, 62, 27, 75, 50, 48, 127,
        127, 49, 51, 90, 34, 70, 99, 56, 24, 82, 62, 27, 75, 50, 48, 127,
        127, 49, 62, 90, 34, 43, 58, 56, 24, 26, 35, 27, 75, 38, 48, 127,
        127, 87, 42, 42, 18, 43, 91, 74, 74, 99, 35, 10, 59, 59, 86, 127,
        127, 87, 42, 42, 18, 94, 91, 68, 52, 99, 102, 10, 59, 59, 86, 127,
        127, 64, 38, 74, 26, 35, 27, 25, 41, 43, 58, 34, 98, 46, 65, 127,
        64, 66, 68, 77, 28, 46, 67, 81, 89, 66, 54, 36, 101, 69, 67, 65,
        64, 66, 68, 36, 28, 69, 67, 65, 64, 66, 68, 36, 28, 69, 67, 65,
        64, 66, 68, 76, 45, 69, 67, 88, 80, 66, 68, 52, 100, 69, 67, 65,
        57, 59, 85, 85, 51, 30, 99, 97, 73, 75, 38, 42, 93, 93, 58, 56,
        57, 59, 61, 85, 53, 30, 58, 97, 73, 59, 38, 44, 93, 60, 58, 56,
        127, 103, 46, 35, 91, 94, 2, 0, 9, 10, 87, 82, 51, 30, 102, 127,
        127, 103, 69, 99, 33, 64, 49, 30, 30, 56, 24, 49, 98, 45, 102, 127,
        127, 103, 69, 99, 56, 24, 49, 43, 43, 56, 40, 32, 98, 45, 102, 127,
        127, 103, 30, 58, 79, 79, 18, 43, 43, 2, 95, 95, 34, 70, 102, 127,
        127, 49, 30, 58, 74, 74, 18, 17, 1, 2, 90, 90, 34, 70, 48, 127,
        127, 49, 51, 60, 68, 52, 50, 17, 1, 51, 53, 60, 36, 50, 48, 127,
        127, 49, 51, 29, 68, 52, 50, 0, 16, 51, 53, 60, 68, 50, 48, 127,
        127, 49, 62, 27, 75, 75, 2, 0, 16, 18, 91, 91, 66, 38, 48, 127,
        127, 87, 62, 27, 78, 78, 2, 42, 42, 18, 94, 94, 66, 38, 86, 127,
        127, 87, 44, 83, 25, 41, 64, 42, 42, 48, 33, 64, 82, 61, 86, 127,
        127, 87, 44, 83, 48, 33, 64, 38, 38, 48, 56, 24, 82, 61, 86, 127,
        127, 87, 38, 50, 99, 102, 10, 9, 1, 2, 79, 74, 26, 46, 86, 127,
        64, 66, 68, 76, 45, 30, 67, 88, 80, 66, 38, 52, 100, 69, 67, 65,
        64, 66, 76, 76, 43, 30, 90, 88, 80, 82, 38, 50, 100, 100, 67, 65,
        57, 59, 82, 82, 51, 30, 99, 17, 17, 75, 38, 42, 90, 90, 58, 56,
        127, 57, 46, 82, 51, 43, 50, 48, 25, 27, 66, 42, 90, 30, 56, 127,
        127, 103, 28, 99, 33, 41, 57, 46, 46, 33, 64, 49, 98, 69, 102, 127,
        127, 103, 45, 99, 97, 127, 96, 98, 99, 97, 127, 96, 98, 28, 102, 127,
        127, 103, 45, 99, 56, 40, 32, 70, 70, 48, 65, 32, 98, 28, 102, 127,
        127, 103, 30, 58, 79, 40, 18, 75, 75, 2, 65, 95, 34, 70, 102, 127,
        127, 49, 30, 91, 66, 42, 27, 25, 57, 59, 67, 58, 74, 70, 48, 127,
        127, 49, 51, 91, 66, 38, 83, 25, 57, 98, 30, 58, 74, 50, 48, 127,
        127, 49, 51, 91, 66, 38, 83, 65, 32, 98, 30, 58, 74, 50, 48, 127,
        127, 49, 62, 91, 66, 59, 67, 65, 32, 34, 43, 58, 74, 38, 48, 127,
        127, 87, 62, 27, 78, 57, 2, 90, 90, 18, 41, 94, 66, 38, 86, 127,
        127, 87, 37, 83, 25, 57, 49, 62, 62, 25, 41, 64, 82, 44, 86, 127,
        127, 87, 37, 83, 81, 127, 80, 82, 83, 81, 127, 80, 82, 44, 86, 127,
        127, 87, 61, 83, 48, 56, 24, 46, 46, 65, 40, 24, 82, 37, 86, 127,
        127, 64, 38, 75, 43, 58, 34, 32, 49, 51, 42, 50, 99, 46, 65, 127,
        64, 66, 75, 75, 43, 30, 90, 16, 16, 82, 38, 50, 99, 99, 67, 65,
        127, 57, 82, 87, 51, 101, 100, 18, 18, 76, 77, 42, 95, 90, 56, 127,
        127, 103, 26, 26, 18, 78, 75, 61, 37, 91, 94, 2, 67, 67, 102, 127,
        127, 103, 28, 99, 48, 25, 57, 26, 26, 33, 41, 57, 98, 69, 102, 127,
        127, 103, 45, 99, 48, 65, 32, 70, 70, 48, 25, 57, 98, 28, 102, 127,
        127, 103, 43, 43, 0, 95, 34, 36, 52, 50, 86, 9, 26, 26, 102, 127,
        127, 103, 43, 43, 2, 42, 90, 54, 38, 83, 67, 10, 26, 26, 102, 127,
        127, 49, 43, 94, 2, 42, 62, 27, 59, 46, 67, 10, 79, 26, 48, 127,
        127, 49, 91, 94, 66, 85, 84, 10, 10, 101, 100, 58, 79, 74, 48, 127,
        127, 49, 91, 94, 66, 85, 84, 10, 10, 101, 100, 58, 79, 74, 48, 127,
        127, 49, 35, 94, 10, 59, 46, 67, 34, 70, 43, 2, 79, 42, 48, 127,
        127, 87, 35, 35, 10, 59, 98, 30, 54, 75, 43, 2, 42, 42, 86, 127,
        127, 87, 35, 35, 9, 103, 51, 53, 29, 27, 78, 1, 42, 42, 86, 127,
        127, 87, 37, 83, 65, 32, 49, 62, 62, 25, 57, 49, 82, 44, 86, 127,
        127, 87, 61, 83, 65, 40, 24, 35, 35, 65, 32, 49, 82, 37, 86, 127,
        127, 87, 59, 59, 2, 79, 74, 28, 69, 90, 95, 18, 35, 35, 86, 127,
        127, 64, 75, 78, 43, 92, 93, 18, 18, 85, 84, 50, 102, 99, 65, 127,
        127, 9, 26, 87, 16, 34, 70, 42, 42, 54, 58, 0, 95, 67, 8, 127,
        127, 103, 26, 26, 18, 27, 75, 82, 82, 91, 43, 2, 67, 67, 102, 127,
        127, 103, 70, 50, 86, 86, 10, 26, 26, 18, 78, 78, 59, 46, 102, 127,
        127, 103, 70, 50, 86, 65, 10, 74, 74, 18, 25, 78, 59, 46, 102, 127,
        127, 103, 43, 43, 2, 67, 90, 54, 38, 83, 27, 10, 26, 26, 102, 127,
        127, 103, 43, 43, 2, 51, 29, 54, 38, 44, 35, 10, 26, 26, 102, 127,
        127, 16, 43, 41, 1, 51, 92, 66, 66, 85, 35, 8, 24, 26, 17, 127,
        127, 16, 18, 94, 64, 82, 35, 10, 10, 26, 99, 56, 79, 18, 17, 127,
        127, 16, 18, 94, 64, 82, 35, 10, 10, 26, 99, 56, 79, 18, 17, 127,
        127, 16, 35, 33, 8, 26, 100, 58, 58, 77, 50, 0, 40, 42, 17, 127,
        127, 87, 35, 35, 10, 26, 45, 30, 54, 36, 50, 2, 42, 42, 86, 127,
        127, 87, 35, 35, 10, 34, 98, 30, 54, 75, 59, 2, 42, 42, 86, 127,
        127, 87, 46, 67, 95, 32, 18, 91, 91, 10, 57, 103, 51, 62, 86, 127,
        127, 87, 46, 67, 95, 95, 18, 35, 35, 10, 103, 103, 51, 62, 86, 127,
        127, 87, 59, 59, 2, 42, 74, 99, 99, 90, 34, 18, 35, 35, 86, 127,
        127, 8, 59, 78, 1, 66, 54, 43, 43, 62, 27, 17, 102, 35, 9, 127,
        127, 9, 26, 24, 16, 34, 77, 42, 42, 93, 58, 0, 65, 67, 8, 127,
        127, 32, 70, 74, 42, 27, 66, 64, 49, 51, 43, 50, 83, 46, 33, 127,
        127, 32, 70, 50, 83, 83, 10, 8, 16, 18, 75, 75, 59, 46, 33, 127,
        127, 32, 70, 75, 59, 67, 58, 56, 40, 42, 27, 66, 82, 46, 33, 127,
        127, 32, 43, 78, 2, 67, 30, 58, 42, 62, 27, 10, 87, 26, 33, 127,
        127, 16, 43, 41, 1, 51, 92, 66, 66, 85, 35, 8, 24, 26, 17, 127,
        127, 16, 18, 41, 1, 51, 91, 94, 87, 82, 35, 8, 24, 18, 17, 127,
        127, 16, 18, 89, 80, 51, 35, 33, 24, 26, 35, 97, 72, 18, 17, 127,
        127, 16, 18, 89, 80, 26, 35, 33, 24, 26, 50, 97, 72, 18, 17, 127,
        127, 16, 18, 33, 8, 26, 99, 102, 79, 74, 50, 0, 40, 18, 17, 127,
        127, 16, 35, 33, 8, 26, 100, 58, 58, 77, 50, 0, 40, 42, 17, 127,
        127, 24, 35, 102, 10, 34, 70, 43, 66, 38, 59, 2, 95, 42, 25, 127,
        127, 24, 46, 99, 58, 34, 43, 41, 64, 66, 59, 67, 90, 62, 25, 127,
        127, 24, 46, 67, 90, 90, 18, 17, 8, 10, 98, 98, 51, 62, 25, 127,
        127, 24, 46, 98, 51, 42, 50, 48, 56, 58, 34, 43, 91, 62, 25, 127,
        127, 8, 59, 57, 1, 66, 76, 43, 43, 92, 27, 17, 33, 35, 9, 127,
        127, 9, 26, 79, 40, 90, 58, 2, 2, 67, 99, 48, 86, 67, 8, 127,
        127, 32, 34, 74, 42, 62, 91, 64, 49, 98, 70, 50, 83, 35, 33, 127,
        127, 32, 34, 52, 61, 37, 35, 8, 16, 34, 36, 68, 61, 35, 33, 127,
        127, 32, 34, 75, 59, 46, 99, 56, 40, 90, 54, 66, 82, 35, 33, 127,
        127, 32, 75, 78, 59, 101, 100, 2, 2, 93, 92, 66, 87, 82, 33, 127,
        127, 16, 18, 78, 57, 98, 43, 2, 2, 51, 91, 64, 87, 18, 17, 127,
        127, 16, 18, 73, 96, 51, 43, 41, 49, 51, 35, 89, 80, 18, 17, 127,
        127, 16, 18, 41, 49, 16, 18, 89, 80, 18, 17, 33, 24, 18, 17, 127,
        127, 16, 18, 33, 24, 16, 18, 97, 72, 18, 17, 48, 40, 18, 17, 127,
        127, 16, 18, 97, 72, 26, 50, 48, 40, 42, 50, 81, 88, 18, 17, 127,
        127, 16, 18, 102, 56, 74, 50, 2, 2, 42, 83, 65, 95, 18, 17, 127,
        127, 24, 99, 102, 58, 77, 76, 2, 2, 85, 84, 67, 95, 90, 25, 127,
        127, 24, 26, 99, 58, 54, 75, 41, 64, 82, 46, 67, 90, 27, 25, 127,
        127, 24, 26, 69, 60, 29, 27, 17, 8, 26, 28, 69, 53, 27, 25, 127,
        127, 24, 26, 98, 51, 62, 83, 48, 56, 74, 70, 43, 91, 27, 25, 127,
        127, 8, 59, 103, 49, 82, 59, 2, 2, 66, 75, 41, 94, 35, 9, 127,
        127, 9, 58, 79, 40, 90, 58, 2, 2, 67, 99, 48, 86, 27, 8, 127,
        127, 32, 34, 74, 42, 62, 91, 41, 57, 98, 70, 50, 83, 35, 33, 127,
        127, 32, 34, 68, 61, 37, 35, 17, 9, 34, 36, 68, 44, 35, 33, 127,
        127, 32, 34, 75, 59, 46, 99, 48, 65, 90, 54, 66, 82, 35, 33, 127,
        127, 32, 75, 78, 59, 101, 100, 2, 2, 93, 92, 66, 87, 82, 33, 127,
        127, 16, 18, 78, 57, 98, 43, 2, 2, 51, 91, 64, 87, 18, 17, 127,
        127, 16, 18, 73, 96, 34, 43, 41, 49, 51, 43, 89, 80, 18, 17, 127,
        127, 16, 18, 25, 32, 16, 18, 73, 96, 18, 17, 41, 49, 18, 17, 127,
        127, 16, 18, 48, 40, 16, 18, 81, 88, 18, 17, 25, 32, 18, 17, 127,
        127, 16, 18, 97, 72, 42, 50, 48, 40, 42, 27, 81, 88, 18, 17, 127,
        127, 16, 18, 102, 56, 74, 50, 2, 2, 42, 83, 65, 95, 18, 17, 127,
        127, 24, 99, 102, 58, 77, 76, 2, 2, 85, 84, 67, 95, 90, 25, 127,
        127, 24, 26, 99, 58, 54, 75, 57, 49, 82, 46, 67, 90, 27, 25, 127,
        127, 24, 26, 45, 60, 29, 27, 9, 16, 26, 28, 69, 60, 27, 25, 127,
        127, 24, 26, 98, 51, 62, 83, 65, 40, 74, 70, 43, 91, 27, 25, 127,
        127, 8, 34, 103, 49, 82, 59, 2, 2, 66, 75, 41, 94, 66, 9, 127,
        127, 9, 58, 56, 0, 67, 100, 50, 50, 84, 35, 17, 25, 27, 8, 127,
        127, 32, 54, 74, 42, 51, 43, 41, 57, 59, 26, 50, 83, 62, 33, 127,
        127, 32, 54, 66, 82, 82, 18, 17, 9, 10, 74, 74, 42, 62, 33, 127,
        127, 32, 54, 75, 59, 26, 50, 48, 65, 67, 58, 66, 82, 62, 33, 127,
        127, 32, 27, 78, 10, 26, 70, 50, 67, 30, 58, 2, 87, 51, 33, 127,
        127, 16, 27, 25, 9, 34, 76, 59, 59, 101, 43, 1, 49, 51, 17, 127,
        127, 16, 18, 25, 9, 34, 75, 78, 103, 98, 43, 1, 49, 18, 17, 127,
        127, 16, 18, 81, 88, 34, 27, 25, 32, 34, 43, 73, 96, 18, 17, 127,
        127, 16, 18, 81, 88, 42, 27, 25, 32, 34, 27, 73, 96, 18, 17, 127,
        127, 16, 18, 48, 0, 42, 83, 86, 95, 90, 27, 9, 32, 18, 17, 127,
        127, 16, 50, 48, 0, 42, 84, 67, 67, 93, 27, 9, 32, 34, 17, 127,
        127, 24, 50, 102, 2, 66, 38, 59, 51, 62, 35, 10, 95, 34, 25, 127,
        127, 24, 70, 99, 58, 66, 59, 57, 49, 51, 35, 67, 90, 54, 25, 127,
        127, 24, 70, 43, 91, 91, 10, 9, 16, 18, 99, 99, 58, 54, 25, 127,
        127, 24, 70, 98, 51, 35, 67, 65, 40, 42, 50, 43, 91, 54, 25, 127,
        127, 8, 34, 32, 16, 26, 101, 51, 51, 85, 59, 1, 64, 66, 9, 127,
        127, 9, 58, 102, 0, 67, 46, 50, 50, 62, 35, 17, 78, 27, 8, 127,
        127, 95, 58, 58, 2, 51, 98, 75, 75, 82, 26, 18, 27, 27, 94, 127,
        127, 95, 54, 66, 87, 87, 18, 27, 27, 10, 79, 79, 42, 62, 94, 127,
        127, 95, 54, 66, 87, 24, 18, 83, 83, 10, 56, 79, 42, 62, 94, 127,
        127, 95, 27, 27, 10, 26, 74, 38, 46, 99, 58, 2, 51, 51, 94, 127,
        127, 95, 27, 27, 10, 34, 52, 38, 46, 28, 43, 2, 51, 51, 94, 127,
        127, 16, 27, 25, 9, 34, 76, 59, 59, 101, 43, 1, 49, 51, 17, 127,
        127, 16, 18, 86, 65, 90, 27, 10, 10, 34, 75, 57, 103, 18, 17, 127,
        127, 16, 18, 86, 65, 90, 27, 10, 10, 34, 75, 57, 103, 18, 17, 127,
        127, 16, 50, 48, 0, 42, 84, 67, 67, 93, 27, 9, 32, 34, 17, 127,
        127, 79, 50, 50, 2, 42, 37, 46, 30, 53, 27, 10, 34, 34, 78, 127,
        127, 79, 50, 50, 2, 66, 82, 46, 30, 91, 35, 10, 34, 34, 78, 127,
        127, 79, 70, 43, 94, 64, 10, 98, 98, 18, 33, 102, 58, 54, 78, 127,
        127, 79, 70, 43, 94, 94, 10, 34, 34, 18, 102, 102, 58, 54, 78, 127,
        127, 79, 34, 34, 18, 35, 99, 90, 90, 83, 50, 2, 66, 66, 78, 127,
        127, 8, 34, 95, 16, 26, 70, 51, 51, 46, 59, 1, 87, 66, 9, 127,
        127, 65, 99, 102, 50, 84, 85, 18, 18, 93, 92, 43, 78, 75, 64, 127,
        127, 95, 58, 58, 2, 103, 98, 36, 68, 82, 87, 18, 27, 27, 94, 127,
        127, 95, 60, 91, 64, 49, 32, 27, 27, 64, 24, 40, 90, 29, 94, 127,
        127, 95, 29, 91, 64, 24, 40, 62, 62, 33, 56, 40, 90, 53, 94, 127,
        127, 95, 27, 27, 8, 79, 42, 44, 37, 35, 102, 0, 51, 51, 94, 127,
        127, 95, 27, 27, 10, 58, 74, 38, 46, 99, 50, 2, 51, 51, 94, 127,
        127, 40, 27, 86, 10, 58, 54, 66, 26, 70, 50, 2, 103, 51, 41, 127,
        127, 40, 83, 86, 67, 93, 92, 10, 10, 77, 76, 59, 103, 98, 41, 127,
        127, 40, 83, 86, 67, 93, 92, 10, 10, 77, 76, 59, 103, 98, 41, 127,
        127, 40, 50, 86, 2, 51, 62, 35, 58, 54, 66, 10, 103, 34, 41, 127,
        127, 79, 50, 50, 2, 51, 82, 46, 30, 91, 66, 10, 34, 34, 78, 127,
        127, 79, 50, 50, 1, 87, 26, 28, 45, 43, 94, 8, 34, 34, 78, 127,
        127, 79, 52, 75, 41, 64, 24, 70, 70, 41, 33, 56, 74, 36, 78, 127,
        127, 79, 36, 75, 41, 33, 56, 34, 34, 25, 48, 56, 74, 68, 78, 127,
        127, 79, 34, 34, 18, 102, 99, 60, 29, 83, 86, 2, 66, 66, 78, 127,
        127, 56, 90, 95, 42, 77, 76, 18, 18, 100, 101, 51, 87, 82, 57, 127,
        65, 67, 99, 99, 50, 38, 82, 16, 16, 90, 30, 43, 75, 75, 66, 64,
        127, 65, 30, 99, 50, 59, 26, 24, 40, 42, 51, 43, 75, 54, 64, 127,
        127, 95, 60, 91, 41, 57, 32, 54, 54, 64, 49, 32, 90, 29, 94, 127,
        127, 95, 29, 91, 89, 127, 88, 90, 91, 89, 127, 88, 90, 53, 94, 127,
        127, 95, 29, 91, 33, 56, 40, 62, 62, 33, 48, 65, 90, 53, 94, 127,
        127, 95, 62, 35, 102, 56, 2, 82, 82, 18, 48, 86, 67, 30, 94, 127,
        127, 40, 62, 83, 67, 58, 66, 64, 24, 26, 50, 59, 98, 30, 41, 127,
        127, 40, 42, 83, 67, 30, 91, 64, 24, 74, 38, 59, 98, 43, 41, 127,
        127, 40, 42, 83, 67, 30, 91, 33, 56, 74, 38, 59, 98, 43, 41, 127,
        127, 40, 38, 83, 67, 51, 35, 33, 56, 58, 66, 59, 98, 70, 41, 127,
        127, 79, 38, 59, 103, 49, 18, 99, 99, 2, 64, 87, 26, 70, 78, 127,
        127, 79, 52, 75, 57, 49, 24, 70, 70, 41, 64, 24, 74, 36, 78, 127,
        127, 79, 52, 75, 73, 127, 72, 74, 75, 73, 127, 72, 74, 36, 78, 127,
        127, 79, 36, 75, 25, 48, 56, 54, 54, 25, 65, 40, 74, 68, 78, 127,
        127, 56, 54, 90, 42, 50, 43, 41, 33, 35, 67, 51, 82, 38, 57, 127,
        56, 58, 90, 90, 42, 38, 75, 17, 17, 99, 30, 51, 82, 82, 59, 57,
        65, 67, 100, 100, 50, 38, 82, 80, 88, 90, 30, 43, 76, 76, 66, 64,
        65, 67, 69, 100, 52, 38, 66, 80, 88, 67, 30, 45, 76, 68, 66, 64,
        127, 95, 30, 43, 75, 78, 10, 8, 0, 2, 103, 98, 34, 54, 94, 127,
        127, 95, 53, 91, 41, 25, 65, 30, 30, 41, 57, 32, 90, 60, 94, 127,
        127, 95, 53, 91, 33, 48, 65, 51, 51, 41, 25, 65, 90, 60, 94, 127,
        127, 95, 62, 35, 102, 102, 2, 51, 51, 18, 86, 86, 67, 30, 94, 127,
        127, 40, 62, 35, 99, 99, 2, 1, 16, 18, 83, 83, 67, 30, 41, 127,
        127, 40, 42, 37, 69, 45, 43, 1, 16, 42, 44, 61, 69, 43, 41, 127,
        127, 40, 42, 61, 69, 45, 43, 17, 0, 42, 44, 61, 28, 43, 41, 127,
        127, 40, 38, 59, 98, 98, 18, 17, 0, 2, 82, 82, 26, 70, 41, 127,
        127, 79, 38, 59, 103, 103, 18, 50, 50, 2, 87, 87, 26, 70, 78, 127,
        127, 79, 68, 75, 57, 32, 40, 50, 50, 57, 49, 24, 74, 52, 78, 127,
        127, 79, 68, 75, 25, 65, 40, 38, 38, 57, 32, 40, 74, 52, 78, 127,
        127, 79, 54, 27, 83, 86, 2, 1, 8, 10, 95, 90, 42, 38, 78, 127,
        56, 58, 60, 93, 44, 38, 59, 73, 97, 58, 30, 53, 85, 61, 59, 57,
        56, 58, 93, 93, 42, 38, 75, 73, 97, 99, 30, 51, 85, 85, 59, 57,
        65, 67, 69, 100, 52, 68, 66, 80, 88, 67, 69, 45, 76, 68, 66, 64,
        65, 67, 69, 28, 36, 68, 66, 64, 65, 67, 69, 28, 36, 68, 66, 64,
        65, 67, 69, 101, 36, 54, 66, 89, 81, 67, 46, 28, 77, 68, 66, 64,
        127, 65, 30, 98, 34, 27, 35, 33, 48, 50, 59, 26, 74, 54, 64, 127,
        127, 95, 51, 51, 18, 86, 83, 69, 45, 75, 78, 10, 58, 58, 94, 127,
        127, 95, 51, 51, 18, 50, 83, 98, 98, 75, 27, 10, 58, 58, 94, 127,
        127, 40, 62, 82, 26, 50, 59, 57, 32, 34, 27, 35, 99, 30, 41, 127,
        127, 40, 42, 82, 26, 70, 75, 57, 32, 90, 62, 35, 99, 43, 41, 127,
        127, 40, 42, 82, 26, 70, 75, 25, 65, 90, 62, 35, 99, 43, 41, 127,
        127, 40, 38, 82, 26, 34, 27, 25, 65, 67, 51, 35, 99, 70, 41, 127,
        127, 79, 66, 66, 10, 34, 90, 83, 83, 98, 51, 18, 50, 50, 78, 127,
        127, 79, 66, 66, 10, 95, 90, 44, 61, 98, 103, 18, 50, 50, 78, 127,
        127, 56, 54, 91, 35, 67, 51, 49, 24, 26, 34, 27, 83, 38, 57, 127,
        56, 58, 60, 92, 37, 46, 59, 96, 72, 58, 54, 29, 84, 61, 59, 57,
        56, 58, 60, 29, 37, 61, 59, 57, 56, 58, 60, 29, 37, 61, 59, 57,
        56, 58, 60, 93, 44, 61, 59, 73, 97, 58, 60, 53, 85, 61, 59, 57,
        65, 67, 69, 70, 70, 68, 66, 18, 18, 67, 69, 70, 70, 68, 66, 64,
        65, 67, 69, 101, 36, 68, 66, 89, 81, 67, 69, 28, 77, 68, 66, 64,
        65, 67, 101, 101, 34, 54, 91, 89, 81, 83, 46, 26, 77, 77, 66, 64,
        65, 67, 98, 98, 34, 54, 91, 17, 17, 83, 46, 26, 74, 74, 66, 64,
        127, 65, 98, 103, 34, 93, 92, 18, 18, 84, 85, 26, 79, 74, 64, 127,
        127, 0, 51, 103, 16, 42, 62, 26, 26, 38, 66, 8, 79, 58, 1, 127,
        127, 0, 51, 49, 16, 42, 85, 26, 26, 77, 66, 8, 56, 58, 1, 127,
        127, 0, 51, 87, 24, 74, 66, 10, 10, 58, 91, 33, 102, 58, 1, 127,
        127, 0, 66, 87, 24, 74, 66, 10, 10, 58, 91, 33, 102, 50, 1, 127,
        127, 0, 66, 64, 8, 58, 92, 35, 35, 100, 43, 17, 48, 50, 1, 127,
        127, 0, 66, 94, 8, 58, 30, 35, 35, 70, 43, 17, 86, 50, 1, 127,
        127, 56, 91, 94, 35, 100, 101, 18, 18, 77, 76, 27, 86, 83, 57, 127,
        56, 58, 91, 91, 35, 46, 98, 16, 16, 74, 54, 27, 83, 83, 59, 57,
        56, 58, 92, 92, 35, 46, 98, 96, 72, 74, 54, 27, 84, 84, 59, 57,
        56, 58, 60, 92, 37, 61, 59, 96, 72, 58, 60, 29, 84, 61, 59, 57,
        56, 58, 60, 62, 62, 61, 59, 18, 18, 58, 60, 62, 62, 61, 59, 57,
    };
} obbData;
#endif

#endif
